; DIGITAL SERVO CONTROLLER		PIC Peripheral Initializers
; ------------------------------------------------------------------------------
; ROUTINES:
;   ports_configure -- configure PIC port
;   adc_configure -- configure ADC to capture server pot (current position)
;   pwm_configure -- configure PWM output to H-BRIDGE
;   timer_configure -- configure the sample timer (determines system bandwidth)

;------------------------------------------------------------------------------
; Configure PWM channel, this PWM channel ouputs to the mosfet H-BRIDGE.
;------------------------------------------------------------------------------
pwm_configure:   movlw	    0x3f	  ; our pwm period for PWM 1 & 2
	         bsf        status,rp0    ; bank [1]    PR2
	         movwf	    PR2             
                 bcf        TRISIO,2      ; make CCP1 output bit     
                 bcf        status,rp0    ; bank [0] 

               MOVLW 50
               MOVWF CCPR1L   ;;;  clrf	CCPR1L            ;

	         clrf	CCPR1H            ;                  
	         bcf	CCP1CON, 5        ; clear the two LSBs of the duty cycle
	         bcf	CCP1CON, 4        ;
		 
	         bcf	T2CON, T2CKPS1    ; Set TMR2 prescale
	         bcf	T2CON, T2CKPS0
             bsf	T2CON, TMR2ON     ; start TMR2 period=3f+1=40h	
	         bsf	CCP1CON, CCP1M3   ; set CCP1 to PWM mode
	         bsf	CCP1CON, CCP1M2   ;
	         bcf	CCP1CON, CCP1M1   ;
	         bcf	CCP1CON, CCP1M0   ;	
	         return
;-------------------------------------------------------------------------------
timer_configure: movf	SERVO_CLOCK,w     ; configure the sample timer
	             movwf	TMR1H             ;
	             clrf	TMR1L		      ; clear timer
                 MOVLW  10H               ; 
                 MOVWF	T1CON		      ; 1:2 PRESCALE   timer1 control register
	             bsf	T1CON, TMR1ON     ; start T1 *************************** 
                 BCF    PIR1,TMR1IF       ; CLEAR timer 1 interrupt flag 
                 BCF    PIR1,TMR2IF       ; CLEAR timer 2 interrupt flag
	             bsf    status,rp0        ; bank [1]
	             bSf	PIE1, TMR1IE	  ; SET timer 1 interrupt ENBflag
                 bSf	PIE1, TMR2IE	  ; SET timer 2 interrupt ENBflag
                BCF     STATUS,RP0       
                 BCF    INTCON,T0IF       ; CLEAR timer 0 interrupt flag
                 BSF    INTCON,T0IE       ; SET timer 0 interrupt ENBflag
                               
                 return
;-------------------------------------------------------------------------------
; Configures the PIC ADC to capture the current servo position
;-------------------------------------------------------------------------------
adc_configure:   bsf      status,RP0      ; select bank 1
                 movlw    51h             ; FOSC/16[8mhz clock] & GP0  [51h]
                 movwf    ANSEL           ; set GP0 to Analog
                 bsf      TRISIO,0        ; insure input bit
                 bcf      status,RP0      ; select bank 0
                 movlw    b'10000001'     ; RJustify, Vdd ref, AN0-000,
                 movwf    ADCON0          ; 
                 bcf	  PIE1, ADIF	  ; clear adc flag
	             bsf	  PIE1, ADIE	  ; enable ADC interrupt
                 return                   ;
;-------------------------------------------------------------------------------
; Configures the ports of the PIC
;-------------------------------------------------------------------------------
ports_configure: clrf     GPIO            ; clear port
                 bsf      status,RP0      ; select bank 1
                 movlw    b'00001011'     ; inputs  - GP0,1,3 pin 7,6,4
                 movwf    TRISIO          ; outputs - GP2,4,5 pin 2,1,5
                 return                   ;
;-------------------------------------------------------------------------------
 
