; DIGITAL SERVO CONTROLLER
; ---------------------------------------------------------------------------------------
 
 
; config file has configurable compile options and the PIC target device
#include "config683a.asm"

; registers file configures the memory register layout of the servo code modules
#include "registers683.asm"

#include "isr683.asm"
#INCLUDE "68316BITS.INC"
#INCLUDE "683UTIL.INC"


;=========================================================
;#ifdef PID_NO_OUTPUT_COEFFICIENT
;#define PID_DEFAULT_KP		0x04
;#define PID_DEFAULT_KI		0x20
;#define PID_DEFAULT_KD		0x20
;#define PID_DEFAULT_KPID	0x01
;#else
;--------------------------------------------------
;#if ADC_10BIT
;#define PID_DEFAULT_KP		0x03     ; 10 bit
;#define PID_DEFAULT_KI		0x20
;#define PID_DEFAULT_KD		0x20
;#define PID_DEFAULT_KPID	0x0a
;#else
#define PID_DEFAULT_KP		0x08 ;06 ;;05   ;;04     ; 8 bit
#define PID_DEFAULT_KI		0x60 ;50 ;;40   ;;20
#define PID_DEFAULT_KD		0x20 ;15 ;;10   ;;01
#define PID_DEFAULT_KPID	0x01 ;02 ;;03   ;;04
;#endif
;--------------------------------------------------
;#endif
;===============================================================
; Set program memory base at reset vector 0x00
	org 0x0000

; STARTUP CODE
	goto restart	; normal reset vector
	goto restart	; firmware programming reset vector (if bootload exists)
	nop
	nop

isr_table
; Interrupt Service Routine for the Servo Controller
; macros are defined in helpers.asm
; save the user context
	ISR_ENTER      ;;;;;ISR_CONTEXT_SAVE

	          ;         timer interrupt jump table
	          ;			INT_REG, INT_FLAG, JUMP_VECTOR
	banksel PIR1

	isr_jumpif	    PIR1, 	TMR1IF, T1INT                        ;;;;sample_timer_isr
	isr_jumpif	    PIR1, 	ADIF, 	adc_conversion_complete_isr	
    isr_jumpif      intcon, GPIF,   INT 
    isr_jumpif      PIR1,   TMR2IF, T2INT
    isr_jumpif      INTCON, T0IF,   T0INT
 
;------------------------------------------------------------------         
#ifdef EEPROM_ASYNC_SAVE
	isr_jumpif	PIR2, 	EEIF,	eeprom_isr
#endif
;------------------------------------------------------------------
exit_isr: 	ISR_EXIT     ;;;;ISR_CONTEXT_RESTORE
	        retfie					; return from interrupt
;============================================================================== 
;;           ISR_ENTER     
;-----------------------------------------------------------------------------
INT:      ;; BTFSS        INTCON,GPIF              ; pin change INT
          ;;  GOTO        T1INT
;-----------------------------------------------------------------------------
p_change:  btfsS        SIGNAL_IN       ;;; GPIO,3 ; SIGNAL IN PIN 4 
            GOTO        L_SIG
H_SIG:     MOVF         GPIO,W                   ; CLR PIN CHANGE
           clrf         TMR1L
           CLRF         TMR1H
           BSF          T1CON,TMR1ON             ; START TIMER 1          
           bcf	        INTCON,GPIF              ; EXT INT           
           MOVF         GPIO,W
           GOTO         EXIT_ISR        
;----------------------------------------------------------------------
L_SIG:     BCF          T1CON,TMR1ON             ; STOP TIMER 1
           MOVF         GPIO,W                   ; CLEAR PIN CHANGE
           movf         TMR1H,w
           movwf        RC_PULSEH
           movf         TMR1L,w
           movwf        RC_PULSEL                ; SAVE SERVO PULSE WIDTH
           BSF          PPM_CAP                  ; NOTIFY MAIN OF CAPTURE 
LX:    	   bcf	        INTCON,GPIF              ; EXT INT             
           MOVF         GPIO,W
           GOTO         EXIT_ISR                 ;;;EXIT_INT
;=====================================================================
T1INT:    ; BTFSS        PIR1,TMR1IF   ;; SERVO PULSE STUCK HI
          ; GOTO         T0INT
           BSF          T1EXPIRED
           BCF          PIR1,TMR1IF    
           GOTO         EXIT_ISR
;---------------------------------------------------------------------  
T0INT:    ; BTFSS        INTCON,T0IF
          ; GOTO         T2INT            
T0CLR:     BCF          INTCON,T0IF
           btfsc	SERVO_STATUS, SSF_OFF
         	goto 	exit_isr

        	bsf		ADCON0, GO		; start conversion
	       goto 	exit_isR

           GOTO         EXIT_ISR
;---------------------------------------------------------------------
T2INT:    ; BTFSS        PIR1,TMR2IF
          ; GOTO         EXIT_INT
           BCF          PIR1,TMR2IF ;
           GOTO         EXIT_ISR
 
;----------------------------------------------------------------------------------
#include "math8.asm"            ; include math subroutines
#include "helpers.asm"           ; some helper macros and routines
#include "math16.asm"

;----------------------------------------------------------------------------------
#ifdef PID_16BIT                   ; include Proportionate-Integral-Derivitive (PID) algorithm
  
  #include "pid16.asm"
#else
  #include "pid8.asm"
#endif
;----------------------------------------------------------------------------------
#include "eeprom.asm"                           ; the eeprom source module
;--------------------------------------------------------------------------------
; Called when an ADC conversion has completed.
;     store the new value into CP & the 2 lsb's, detect overrun conditions
;--------------------------------------------------------------------------------
adc_conversion_complete_isr:      ; RIGHT JUSTIFIED
	bcf		PIR1, ADIF	          ; clear ADC interrupt flag
;--------------------------------------------------------------------------------	         	
    movf	ADRESH,w      ; save sample as current position device supports 10bit ADC        
    movwf	SERVO_CPH     ; 2 msb bit 1,0
	  banksel	ADRESL
	  movf	ADRESL,w
	  banksel	SERVO_CP
	  MOVWF SERVO_CP
;---------------------------------------------------------------------------------------
	btfsc	SERVO_STATUS, SSF_PIDRUN   ; IS previous PID loop still calculating?
	goto	servo_overrun		       ; overrun condition!	
	bsf	SERVO_STATUS, SSF_PIDRUN       ; set PID loop to run

  

	goto	exit_isr
;----------------------------------------------------------------------------------------------
; the PID loop took to long and is already running while we are ready for the next sample. Keep
; a total of the number of overruns. We can optionally automatically adjust the clock higher to 
; so this wont happen. In any case, this iteration will be skipped.
;----------------------------------------------------------------------------------------------
servo_overrun:
	incf	SERVO_OVERRUNS_L,f
	btfsc	STATUS,Z
	incf	SERVO_OVERRUNS_H,f
;------------------------------------------------------------------------------
#ifdef AUTO_ADJUST_BANDWIDTH
	incf	SERVO_CLOCK,f
#else
	; only adjust bandwidth if servo option is set
	btfsc	SERVO_OPTIONS, SO_AUTO_ADJUST_BANDWIDTH
	incf	SERVO_CLOCK,f
#endif
	goto	exit_isr

;------------------------------------------------------------------------------

restoreSettings:
	 
	banksel SERVO_DP                     ; clear performance counters
	movlw	0x7f
	movwf	SERVO_DP
	movlw	0xff
	movwf	SERVO_SPEED

	
	clrf	SERVO_STATUS                 ; reset servo status register 
;-----------------------------------------------------------------------------
#ifdef DISABLE_OUTPUT_ON_STARTUP
	bcf	SERVO_STATUS, SSF_OE	     ; disable servo output by default
#else
	bsf	SERVO_STATUS, SSF_OE	     ; enable servo output by default
#endif	
;-----------------------------------------------------------------------------	
	clrf	SERVO_OPTIONS                ; clear servo options register
;-----------------------------------------------------------------------------	
	movlw	SAMPLETIMER_RELOAD_DEFAULT   ; reset sample clock freq
	movwf	SERVO_CLOCK
;-----------------------------------------------------------------------------	
	movlf	PID_DEFAULT_KP, PID_KP       ; set PID coefficients
	movlf	PID_DEFAULT_KI, PID_KI
	movlf	PID_DEFAULT_KD, PID_KD
	movlf	PID_DEFAULT_KPID, PID_KPID
;-----------------------------------------------------------------------------	
	clrf	SERVO_OVERRUNS_L             ; clear performance counters
	clrf	SERVO_OVERRUNS_H
	clrf	SERVO_ITER_L
	clrf	SERVO_ITER_H
	retlw 	0x00
;#########################################################################
; our initialization routines are defined here
#include "pic-init683ff.asm"
;=========================================================================
restart:   ; Power-on start procedure, also call to reset the device.

       bsf	STATUS,RP0                      ; Register bank 1   
             movlw     0x71                 ; use int osc
             movwf     osccon               ; set 8 mhz internal
             clrf      osctune              ; 
       bCf	STATUS,RP0                       

       banksel  OPTION_REG                  ; SETUP TMR0
        MOVLW   0X01                        ; 8 MHZ CLOCK PRESCALE 1:4
    	MOVWF	OPTION_REG                  ; 1 USEC 	TMR0  256US INT  
	              
;             movlw	   b'01010001'
;		      movwf	   ANSEL		        ; Analog I/O on GP0
;--------------------------------------------------------
;  PWM period = [(PR2)+1]*4*Tosc*TMR2 prescale value
;  PWM period - 100*4*.5*1= 200us  Freq= 5khz
;  PWM period - 100*4*.5*4= 800us  Freq= 1.25khz
;--------------------------------------------------------
;              movlw     0x63                ; lets try 63 +1 - 64 = 100 % 
;              movwf     PR2                 ; load period reg ?????????
;
;              bcf       TRISIO,2            ; GP2 = output
;
; 		      movlw	    b'01010001';
;		      movwf	    ANSEL		        ; Analog I/O on GP0
;	          bcf	    STATUS,RP0          ; register bank 0
;
;              MOVLW     0X64/2
;
;              MOVWF     CCPR1L              ;**************************************
;              MOVWF     PCNT
;
;              movlw     b'00001100'         ; 0 0 0 0 0 1 0 0               
;                                            ;  / / / /   \ \ \ 
;              movwf     T2CON               ;postScale    \ 00= 1:1 prescale 
;                                            ;    1:1      start timer;
;
;              movlw     0x0c
;              movwf     CCP1CON              ; select pwm 
;            ;; clrf      CCPR1L              ; set stopped
;
;
	; clear interrupt registers
	banksel	INTCON
	clrf	INTCON
	banksel PIE1
	clrf	PIE1
	;;clrf	PIE2
	;;clrf	SERVO_IDL
	;;clrf	SERVO_IDH

	; reset to servo defaults
	call	restoreSettings

	; configure PIC peripherals
	call 	pid_init
	call	ports_configure
	call	adc_configure
	call	pwm_configure
	call	timer_configure
	;;call	iic_configure
	call	eeprom_configure

	; call testbench if enabled
#ifdef TESTBENCH
#include "testbench.asm"
#endif

	 
	 
	banksel INTCON
	;;bsf	INTCON, INTE		   ; enable interrupt on RB0/INT
	bsf	INTCON, PEIE		   ; enable peripheral interrupts
	bsf	INTCON, GIE		       ; enable global interrupts

    BANKSEL IOC
    BSF     IOC,3
    BANKSEL INTCON
    bsf      intcon,GPIE       ; enable GPIO change int  



	;call eeprom_write_async
	; load settings from eeprom
#ifdef EEPROM_LOAD_ON_STARTUP
	call	loadSettings
#endif

	; enter pid loop!
;###########################################################################################

pid_loop
	; reset run flag
	bcf		SERVO_STATUS, SSF_PIDRUN

pid_loop_idle
	; we must wait for SSF_PIDRUN to be set
	btfss	SERVO_STATUS, SSF_PIDRUN
	 goto 	pid_loop_idle
	clrwdt
	; increment servo iteration counter

    BTFSS   ppm_cap            ; HAVE WE GOT A SERVO PULSE 
     goto 	pid_loop_idle      ; NOT YET
	incf	SERVO_ITER_L,f
	btfsc	STATUS,Z
	incf	SERVO_ITER_H,f

	; hit the watchdog. We put this here to catch ISR or PIDLOOP freeze. If  interupts
	; freeze, PIDRUN would not get set to run PID loop. If a PIDLOOP freezes, PIDRUN
	; would not get cleared so the interrupt wound not start another.

;-------------------------------------------------------------------------------
	; calculate Proportional Error (PE = desired_position - current_position)
    NOP
    MOV16    SERVO_DP,SERVO_E
    SUBI16   SERVO_E,1000
        skpNc                        ;
              GOTO   POK                  ;  STILL > 0
            movi16   12,SERVO_E           ;  < 0 SET MIN VALUE = 0
pok:     nop
            cmpi16   SERVO_E,1023             ;  RCL > 1000 
             skpc                         ;
              GOTO   POK1                 ;  STILL < 1000
            movi16   1023,SERVO_E             ;  >1000 set max pulse =1000
POK1:   nop
            SUBR16   SERVO_E,SERVO_CP     ; DP - CP ==> E
            btfss    status,c
             goto    neg_value
pos_value:

            bcf  SERVO_STATUS, SSF_ENEG
            BCF  STATUS,C      ; CONVERT TO 8 BIT 
            RRF  SERVO_EH,F
            RRF  SERVO_E,F
            BCF  STATUS,C
            RRF  SERVO_EH,F
            RRF  SERVO_E,F
            goto convert_pe
         
neg_value:  neg16 SERVO_E
            BCF  STATUS,C      ; CONVERT TO 8 BIT 
            RRF  SERVO_EH,F
            RRF  SERVO_E,F
            BCF  STATUS,C
            RRF  SERVO_EH,F
            RRF  SERVO_E,F
            bsf		SERVO_STATUS, SSF_ENEG
convert_pe:    
	btfss	SERVO_STATUS, SSF_OE         ; return to idle if output is disabled 
	goto	stop_servo					; return to idle

	; the pid algorithm expects P to be a signed 8bit int. We will simply chop the 8th bit,
	; and then encode based on ESIGN flag. We want to maintain the resolution at small values
	; of E, not concerned with large values of E>127.
	; copy E to P
    NOP
	movff	SERVO_E, PID_P
    NOP
;------------------------------------------------------------------------------------------	
;#ifdef ADRESL         ; if enabled, shift two lsb of ADC (ADRESL) into Proportional Error
;  #ifdef ADC_10BIT
;	
;	maxl	PID_P, 0x1f  ; make 0<=P<=127
;
;	banksel	ADRESL
;	movf	ADRESL,w
;	banksel PID_TEMP
;	movwf	PID_TEMP
;	rlf		PID_TEMP,f
;	rlf		PID_P,f
;	rlf		PID_TEMP,f
;	rlf		PID_P,f
 ; #else
;;make 0<=P<=127
	movlw	0x7f		; if bit P[7] is set, we will max P at 127
	btfsc	PID_P,7
	movwf	PID_P		; max P at 127
 ; #endif
;#endif
;--------------------------------------------------------------------------------------------
	
	btfss	SERVO_STATUS, SSF_ENEG ; if ESIGN not set, we can now calc pid
	goto	calc_pid

	; compliment P back to neg for PID calculation
	comf	PID_P,f
	incf	PID_P,f	

calc_pid
	; calculate PID
	call	pid_calculate
    movf	PID_RESULT,W
	; determine PID polarity
	movf	PID_RESULT,f	; so we can test Z
	btfsc	STATUS, Z
	goto	stop_servo
	btfsC	PID_RESULT, 7
	goto	ccw_servo
;----------------------------------------------------------------
cw_servo:
	 BCF    CCWDRV
#ifdef SPEED_LIMITER
	mov8	PID_RESULT, PID_TEMP
	maxf	PID_TEMP, SERVO_SPEED
    movf	PID_TEMP,w
#endif
     BSF    CWDRV
  	 
     movwf	CCPR1L		; set  clockwise PWM
	 goto pid_loop
;---------------------------------------------------------------
ccw_servo:
	BCF     CWDRV         ; supress clockwise PWM	
	comf	PID_RESULT,w  ; get absolute value of PID
	movwf	PID_TEMP
	incf	PID_TEMP,f
#ifdef SPEED_LIMITER
	maxf	PID_TEMP, SERVO_SPEED
#endif
     movf	PID_TEMP,w
	 BSF    CCWDRV
     movwf	CCPR1L		; set counter clockwise PWM
	goto pid_loop
;-----------------------------------------------------------------
stop_servo:	 
	clrf	CCPR1L      ; no PWM output
	 BCF    CWDRV
     BCF    CCWDRV 
	goto pid_loop
;--------------------------------------------------------------------
	END




